from django.test import TestCase
from django.db import IntegrityError
from django.core.exceptions import FieldDoesNotExist, ObjectDoesNotExist
from tests.models import TestModel
from unittest.mock import patch


class TestLisanModelMixin(TestCase):
    """
    Test suite for the LisanModelMixin class, which provides multilingual
    support for Django models.
    """

    def setUp(self):
        """
        Set up the test environment. Create a test model instance and
        translations for testing.
        """
        self.instance = TestModel.objects.create(
            title="Hello World",
            description="Sample description"
        )
        self.instance.set_lisan('am', title="ሰላም ለዓለም", description="ምሳሌ መግለጫ")
        self.instance.set_lisan('tg', title="ሰላም ዓለም")

    def test_get_lisan_existing_language(self):
        """
        Test retrieving a translation (Lisan) for an existing language.
        """
        lisan = self.instance.get_lisan('am')
        self.assertIsNotNone(lisan)
        self.assertEqual(lisan.title, "ሰላም ለዓለም")
        self.assertEqual(lisan.description, "ምሳሌ መግለጫ")

    def test_get_lisan_nonexistent_language(self):
        """
        Test retrieving a translation (Lisan) for a non-existent language.
        """
        lisan = self.instance.get_lisan('fr')
        self.assertIsNone(lisan)

    def test_set_lisan_new_language(self):
        """
        Test setting a translation for a new language.
        """
        lisan = self.instance.set_lisan(
            'or', title="Nagaa Addunyaa", description="Miseensa fakkeessaa")
        self.assertEqual(lisan.title, "Nagaa Addunyaa")
        self.assertEqual(lisan.description, "Miseensa fakkeessaa")

    def test_set_lisan_update_existing(self):
        """
        Test updating an existing translation for a language.
        """
        self.instance.set_lisan('am', title="አዲስ ሰላም")
        lisan = self.instance.get_lisan('am')
        self.assertEqual(lisan.title, "አዲስ ሰላም")
        self.assertEqual(lisan.description, "ምሳሌ መግለጫ")  # Unchanged field

    def test_set_lisan_invalid_field(self):
        """
        Test setting a translation with a field that does not exist.
        """
        with self.assertRaises(FieldDoesNotExist):
            self.instance.set_lisan('am', non_existent_field="Invalid")

    def test_set_lisan_invalid_language_code(self):
        """
        Test setting a translation with an unsupported language code.
        """
        with self.assertRaises(ValueError):
            self.instance.set_lisan('xx', title="Unsupported Language")

    def test_set_bulk_lisans(self):
        """
        Test setting translations in bulk for multiple languages.
        """
        translations = [
            {
                "language_code": "or",
                "title": "Nagaa Addunyaa",
                "description": "Fakkeenya"
            },
            {
                "language_code": "am",
                "title": "አስደሳች ጉዞ",
                "description": "በማይታወቁ መሬቶች ላይ አስገራሚ ጉዞ."
            }
        ]
        self.instance.set_bulk_lisans(translations)

        am_translation = self.instance.get_lisan('am')
        or_translation = self.instance.get_lisan('or')

        self.assertEqual(am_translation.title, "አስደሳች ጉዞ")
        self.assertEqual(or_translation.title, "Nagaa Addunyaa")
        self.assertEqual(or_translation.description, "Fakkeenya")

    def test_get_lisan_field_existing_language(self):
        """
        Test retrieving a field's value for an existing translation.
        """
        title = self.instance.get_lisan_field('title', 'am')
        self.assertEqual(title, "ሰላም ለዓለም")

    def test_get_lisan_field_with_fallback(self):
        """
        Test retrieving a field's value with fallback languages.
        """
        title = self.instance.get_lisan_field(
            'title', 'fr', fallback_languages=['tg'])
        self.assertEqual(title, "ሰላም ዓለም")

    def test_get_lisan_field_auto_translate(self):
        """
        Test retrieving a field's value with auto-translation enabled.
        """
        with patch('lisan.utils.get_translation_service') as mock_service:
            mock_service.return_value = "Bonjour le monde"  # noqa
            title = self.instance.get_lisan_field(
                'title', 'fr', auto_translate=True)
            self.assertEqual(title, f"{self.instance.title} in fr")

    def test_set_current_language(self):
        """
        Test setting the current language for the model instance.
        """
        self.instance.set_current_language('am')
        self.assertEqual(self.instance._current_language, 'am')

    def test_set_invalid_current_language(self):
        """
        Test setting an unsupported current language.
        """
        with self.assertRaises(ValueError):
            self.instance.set_current_language('xx')

    def test_is_field_translatable(self):
        """
        Test checking if a field is translatable.
        """
        self.assertTrue(self.instance.is_field_translatable('title'))
        self.assertFalse(
            self.instance.is_field_translatable('non_translatable_field'))

    def test_validate_language_code(self):
        """
        Test validating a supported language code.
        """
        # Should not raise an exception
        self.instance._validate_language_code('am')

    def test_validate_invalid_language_code(self):
        """
        Test validating an unsupported language code.
        """
        with self.assertRaises(ValueError):
            self.instance._validate_language_code('xx')

    def test_get_lisan_object_does_not_exist(self):
        """
        Test that get_lisan returns None if the Lisan object does not exist.
        """
        with patch.object(
                self.instance.Lisan.objects,
                'filter',
                side_effect=ObjectDoesNotExist):
            lisan = self.instance.get_lisan('am')
            self.assertIsNone(lisan)

    def test_get_lisan_general_exception(self):
        """
        Test that get_lisan raises a general exception if one occurs.
        """
        with patch.object(
                self.instance.Lisan.objects,
                'filter',
                side_effect=Exception("Unexpected error")):
            with self.assertRaises(Exception) as context:
                self.instance.get_lisan('am')
            self.assertEqual(str(context.exception), "Unexpected error")

    def test_set_lisan_field_does_not_exist(self):
        """
        Test that set_lisan raises FieldDoesNotExist if an invalid
        field is provided.
        """
        with self.assertRaises(FieldDoesNotExist) as context:
            self.instance.set_lisan('am', non_existent_field="Invalid")
        self.assertEqual(
            str(context.exception),
            "Field 'non_existent_field' does not exist in the translation model."  # noqa
        )

    def test_set_lisan_primary_key_field(self):
        """
        Test that set_lisan handles a primary key field that is not named 'id'.
        """
        # Create a Lisan translation with a custom primary key
        lisan = self.instance.set_lisan(
            'am', title="ሰላም", description="ምሳሌ"
        )
        self.assertEqual(lisan.language_code, "am")
        self.assertEqual(lisan.title, "ሰላም")
        self.assertEqual(lisan.description, "ምሳሌ")
        self.assertEqual(lisan.testmodel_id, self.instance.id)

    def test_set_lisan_primary_key_value(self):
        """
        Test that set_lisan sets the primary key value when provided.
        """
        lisan = self.instance.set_lisan(
            'am', title="ሰላም", description="ምሳሌ"
        )
        self.assertEqual(lisan.language_code, "am")
        self.assertEqual(lisan.testmodel_id, self.instance.id)

    def test_set_lisan_replaces_existing_translation(self):
        """
        Test that set_lisan replaces the existing translation for a language
        instead of creating a duplicate.
        """
        # Update the 'am' translation
        self.instance.set_lisan('am', title="Updated Title")

        # Retrieve the updated translation
        lisan = self.instance.get_lisan('am')

        # Verify the translation was updated
        self.assertEqual(lisan.title, "Updated Title")
        self.assertEqual(lisan.description, "ምሳሌ መግለጫ")  # Unchanged field

        # Verify that no duplicate translations exist
        am_translations_count = self.instance.Lisan.objects.filter(
            language_code='am').count()
        self.assertEqual(am_translations_count, 1)

    def test_set_lisan_creates_new_translation(self):
        """
        Test that set_lisan creates a new translation if one does not exist
        for the specified language.
        """
        # Add a new translation for 'or' (Oromo)
        lisan = self.instance.set_lisan(
            'or', title="Nagaa Addunyaa", description="Fakkeenya")

        # Verify the translation was created
        self.assertEqual(lisan.language_code, "or")
        self.assertEqual(lisan.title, "Nagaa Addunyaa")
        self.assertEqual(lisan.description, "Fakkeenya")

    def test_set_bulk_lisans_updates_and_creates_translations(self):
        """
        Test that set_bulk_lisans updates existing translations and creates
        new translations as needed.
        """
        translations = [
            {
                "language_code": "or",
                "title": "Nagaa Addunyaa",
                "description": "Fakkeenya"
            },
            {
                "language_code": "am",
                "title": "አስደሳች ጉዞ",
                "description": "ምሳሌ መግለጫ"
            }
        ]
        self.instance.set_bulk_lisans(translations)

        # Verify the 'am' translation was updated
        am_translation = self.instance.get_lisan('am')
        self.assertEqual(am_translation.title, "አስደሳች ጉዞ")
        # Unchanged field
        self.assertEqual(am_translation.description, "ምሳሌ መግለጫ")

        # Verify the 'or' translation was created
        or_translation = self.instance.get_lisan('or')
        self.assertEqual(or_translation.title, "Nagaa Addunyaa")
        self.assertEqual(or_translation.description, "Fakkeenya")

        # Verify that no duplicates exist
        am_translations_count = self.instance.Lisan.objects.filter(
            language_code='am').count()
        or_translations_count = self.instance.Lisan.objects.filter(
            language_code='or').count()
        self.assertEqual(am_translations_count, 1)
        self.assertEqual(or_translations_count, 1)



from django.test import TestCase, RequestFactory
from django.conf import settings
from unittest.mock import MagicMock, Mock
from lisan.middleware import LanguageMiddleware


class TestLanguageMiddleware(TestCase):
    """
    Test suite for the LanguageMiddleware class, which sets the language
    code on the request object based on various input sources.
    """

    def setUp(self):
        """
        Set up the test environment, including the middleware instance
        and request factory.
        """
        self.factory = RequestFactory()
        self.middleware = LanguageMiddleware(get_response=Mock())

        # Set test settings for language configuration
        settings.LISAN_DEFAULT_LANGUAGE = 'en'
        settings.LISAN_ALLOWED_LANGUAGES = ['en', 'am', 'or', 'tg']

    def test_language_from_get_parameter(self):
        """
        Test that the middleware extracts the language code from the 'lang'
        GET parameter.
        """
        request = self.factory.get('/?lang=am')
        self.middleware.process_request(request)
        self.assertEqual(request.language_code, 'am')

    def test_language_from_user_profile(self):
        """
        Test that the middleware extracts the language code from the user's
        profile preference if available.
        """
        mock_user = MagicMock()
        mock_user.profile.language_preference = 'or'
        request = self.factory.get('/')
        request.user = mock_user

        self.middleware.process_request(request)
        self.assertEqual(request.language_code, 'or')

    def test_language_from_cookie(self):
        """
        Test that the middleware extracts the language code from cookies.
        """
        request = self.factory.get('/')
        request.COOKIES['language'] = 'tg'

        self.middleware.process_request(request)
        self.assertEqual(request.language_code, 'tg')

    def test_language_from_accept_language_header(self):
        """
        Test that the middleware extracts the language code from the
        'Accept-Language' header.
        """
        request = self.factory.get('/', HTTP_ACCEPT_LANGUAGE='am, en;q=0.8')
        self.middleware.process_request(request)
        self.assertEqual(request.language_code, 'am')

    def test_fallback_to_default_language(self):
        """
        Test that the middleware falls back to the default language when no
        language source is available or valid.
        """
        request = self.factory.get('/')
        self.middleware.process_request(request)
        self.assertEqual(request.language_code, 'en')

    def test_unsupported_language(self):
        """
        Test that the middleware falls back to the default language when an
        unsupported language is specified.
        """
        request = self.factory.get('/?lang=unsupported_language')
        self.middleware.process_request(request)
        self.assertEqual(request.language_code, 'en')

    def test_parse_accept_language_valid_header(self):
        """
        Test that parse_accept_language correctly extracts the primary language
        from a valid 'Accept-Language' header.
        """
        result = self.middleware.parse_accept_language('am, en;q=0.8')
        self.assertEqual(result, 'am')

    def test_parse_accept_language_empty_header(self):
        """
        Test that parse_accept_language returns None for an empty or missing
        'Accept-Language' header.
        """
        result = self.middleware.parse_accept_language('')
        self.assertIsNone(result)

        result = self.middleware.parse_accept_language(None)
        self.assertIsNone(result)

    def test_parse_accept_language_malformed_header(self):
        """
        Test that parse_accept_language gracefully handles malformed headers.
        """
        result = self.middleware.parse_accept_language(';;;')
        self.assertIsNone(result)

    def test_language_precedence(self):
        """
        Test that the middleware applies the correct precedence when
        multiple language sources are available.
        """
        mock_user = MagicMock()
        mock_user.profile.language_preference = 'or'
        request = self.factory.get('/?lang=am', HTTP_ACCEPT_LANGUAGE='tg')
        request.user = mock_user
        request.COOKIES['language'] = 'tg'

        # GET parameter should take precedence
        self.middleware.process_request(request)
        self.assertEqual(request.language_code, 'am')

    def test_parse_accept_language_no_languages_extracted(self):
        """
        Test that parse_accept_language returns None when the header
        is provided but no valid languages can be extracted.
        """
        # Header with only commas (no valid languages)
        result = self.middleware.parse_accept_language(',,,')
        self.assertIsNone(result)

        # Header with malformed values that result in empty language codes
        result = self.middleware.parse_accept_language(' , ; , ; ')
        self.assertIsNone(result)



from django.conf import settings
from rest_framework import serializers
from rest_framework.exceptions import ValidationError


class TranslationSerializer(serializers.Serializer):
    """
    Serializer for each translation entry, containing a
    language code and the translatable fields.
    """
    language_code = serializers.CharField()

    def __init__(self, *args, **kwargs):
        # Dynamically add fields based on `lisan_fields`
        # provided by the main model
        lisan_fields = kwargs.pop('lisan_fields', [])
        super().__init__(*args, **kwargs)

        for field in lisan_fields:
            self.fields[field] = serializers.CharField(
                allow_blank=True, required=False)


class LisanSerializerMixin(serializers.ModelSerializer):
    """
    A serializer mixin that handles dynamic language translations
    for models that support multi-language fields. This mixin
    adds support for handling translations in POST, PUT, and PATCH
    requests, as well as customizing the serialization output
    based on the requested language.
    """

    def __init__(self, *args, **kwargs):
        """
        Initialize the serializer, setting up the request context,
        allowed languages, and default language. Also handles the
        addition or removal of dynamic fields based on the request method.
        """
        super().__init__(*args, **kwargs)
        self.request = self.context.get('request', None)
        self.allowed_languages = getattr(
            settings, 'LISAN_ALLOWED_LANGUAGES', ['en']
        )
        self.default_language = getattr(
            settings, 'LISAN_DEFAULT_LANGUAGE', 'en'
        )

        # Initialize `translations` with the nested
        # serializer for each language entry
        self.fields['translations'] = serializers.ListSerializer(
            child=TranslationSerializer(
                lisan_fields=getattr(self.Meta.model, 'lisan_fields', [])),
            required=False,
            write_only=True
        )

        self._handle_dynamic_fields()

    def _handle_dynamic_fields(self):
        """
        Handle the inclusion or exclusion of the 'translations' field
        in the serializer based on the request method. The field is
        added for POST, PUT, and PATCH requests and removed otherwise.
        """
        if self.request and self.request.method in ['POST', 'PUT', 'PATCH']:
            if 'translations' not in self.fields:
                self.fields['translations'] = serializers.ListSerializer(
                    child=TranslationSerializer(
                        lisan_fields=getattr(
                            self.Meta.model, 'lisan_fields', [])),
                    required=False,
                    write_only=True
                )
        else:
            self.fields.pop('translations', None)

    def to_representation(self, instance):
        """
        Override the default representation of the instance to include
        language-specific fields based on the requested language.
        """
        representation = super().to_representation(instance)
        language_code = getattr(
            self.request, 'language_code', self.default_language
        )

        # Ensure the language code is within the allowed languages
        if language_code not in self.allowed_languages:
            language_code = self.default_language

        # Modify the representation to include language-specific fields
        if hasattr(instance, 'lisan_fields'):
            for field in instance.lisan_fields:
                if field in representation:
                    representation[field] = instance.get_lisan_field(
                        field, language_code
                    )

        # Add structured `translations` with data for each language
        translations_representation = []
        for lang_code in self.allowed_languages:
            translation_data = {'language_code': lang_code}
            for field in instance.lisan_fields:
                translation_data[field] = instance.get_lisan_field(
                    field, lang_code)
            translations_representation.append(translation_data)

        representation['translations'] = translations_representation
        return representation

    def create(self, validated_data):
        """
        Create a new instance of the model, handling any translations
        provided in the validated data. The translations are processed
        and saved separately after the main instance is created.
        """
        translations = validated_data.pop('translations', [])
        self._validate_translations(translations)
        language_code = getattr(
            self.request, 'language_code', self.default_language
        )

        # Ensure the language code is within allowed languages,
        if language_code not in self.allowed_languages:
            # eventhought it's hard to reach here
            language_code = self.default_language

        # Create the main instance
        instance = super().create(validated_data)
        instance.set_bulk_lisans(translations)

        return instance

    def update(self, instance, validated_data):
        """
        Update an existing instance of the model, handling any translations
        provided in the validated data. The translations are processed and
        saved separately after the main instance is updated.
        """
        translations = validated_data.pop('translations', [])
        self._validate_translations(translations, partial=True)
        language_code = getattr(
            self.request, 'language_code', self.default_language
        )

        # Ensure the language code is within allowed languages
        if language_code not in self.allowed_languages:
            # eventhought it's hard to reach here
            language_code = self.default_language

        # Track translatable fields updated in the main model
        translatable_updates = {
            field: validated_data[field]
            for field in self.Meta.model.lisan_fields
            if field in validated_data
        }

        # Update the main instance with non-translation fields
        instance = super().update(instance, validated_data)

        # Synchronize updated translatable fields with the default
        # language translation
        if translatable_updates:
            instance.set_lisan(language_code, **translatable_updates)

        # Process and save the translations
        for translation in translations:
            lang_code = translation.get('language_code', language_code)

            # Only include fields that are present in the translation
            lisan_fields = {
                field: translation[field]
                for field in self.Meta.model.lisan_fields
                if field in translation
            }

            if lisan_fields:
                instance.set_lisan(lang_code, **lisan_fields)

        return instance

    def _validate_translations(self, translations, partial=False):
        """
        Validate that the translations contain all required languages and
        fields. This method ensures that each translation includes a
        language_code and that all necessary fields are present for each
        translation.
        """
        if not translations:
            # Skip translation validation for partial updates
            # if no translation data is provided
            if not partial:
                raise ValidationError("Translations are required.")
            return

        translation_languages = {
            translation['language_code'] for translation in translations
        }
        missing_languages = set(self.allowed_languages) - translation_languages

        if missing_languages and not partial:
            raise ValidationError(
                "Missing translations for languages: "
                f"{', '.join(missing_languages)}"
            )

        for translation in translations:
            lang_code = translation.get('language_code')
            if lang_code not in self.allowed_languages:
                raise ValidationError(
                    f"Unsupported language code: {lang_code}"
                )

            if not partial:
                missing_fields = set(
                    self.Meta.model.lisan_fields
                ) - translation.keys()
                if missing_fields:
                    raise ValidationError(
                        f"Missing fields for {lang_code}: "
                        f"{', '.join(missing_fields)}"
                    )



from django.test import TestCase
from rest_framework.test import APIRequestFactory
from rest_framework.exceptions import ValidationError
from tests.models import TestModel
from lisan.serializers import TranslationSerializer, LisanSerializerMixin


class TestTranslationSerializer(TestCase):
    """
    Test suite for the TranslationSerializer class.
    """

    def test_dynamic_fields(self):
        """
        Test that TranslationSerializer dynamically adds fields based on
        `lisan_fields`.
        """
        serializer = TranslationSerializer(
            lisan_fields=['title', 'description'])
        self.assertIn('title', serializer.fields)
        self.assertIn('description', serializer.fields)
        self.assertIn('language_code', serializer.fields)

    def test_serialize_translation(self):
        """
        Test serialization of translation data.
        """
        data = {'language_code': 'am', 'title': 'ሰላም', 'description': 'ምሳሌ'}
        serializer = TranslationSerializer(
            lisan_fields=['title', 'description'], data=data)
        self.assertTrue(serializer.is_valid())
        self.assertEqual(serializer.validated_data, data)

    def test_deserialize_translation(self):
        """
        Test deserialization of translation data.
        """
        serializer = TranslationSerializer(
            lisan_fields=['title', 'description'])
        instance = serializer.to_representation({
            'language_code': 'am', 'title': 'ሰላም', 'description': 'ምሳሌ'
        })
        self.assertEqual(instance['language_code'], 'am')
        self.assertEqual(instance['title'], 'ሰላም')
        self.assertEqual(instance['description'], 'ምሳሌ')


class TestLisanSerializerMixin(TestCase):
    """
    Test suite for the LisanSerializerMixin class.
    """

    def setUp(self):
        """
        Set up the test environment with a mock request and model instance.
        """
        self.factory = APIRequestFactory()
        self.request = self.factory.get('/')
        self.request.language_code = 'en'
        self.model_instance = TestModel.objects.create(
            title="Hello World",
            description="Sample description"
        )
        self.model_instance.set_lisan('am', title="ሰላም", description="ምሳሌ")

        class TestModelSerializer(LisanSerializerMixin):
            class Meta:
                model = TestModel
                fields = ['id', 'title', 'description', 'author']

        self.serializer_class = TestModelSerializer

    def test_representation(self):
        """
        Test the representation of the model with language-specific fields
        and structured translations.
        """
        serializer = self.serializer_class(
            self.model_instance, context={'request': self.request})
        representation = serializer.data

        # Check language-specific fields
        self.assertEqual(representation['title'], "Hello World")
        self.assertEqual(representation['description'], "Sample description")

        # Check structured translations
        self.assertIn('translations', representation)
        translations = representation['translations']
        self.assertEqual(len(translations), 4)  # en, am, or, tg
        self.assertEqual(translations[1]['language_code'], 'am')
        self.assertEqual(translations[1]['title'], "ሰላም")
        self.assertEqual(translations[1]['description'], "ምሳሌ")

    def test_create_with_translations(self):
        """
        Test creating a model instance with translations.
        """
        self.request.method = 'POST'
        data = {
            'title': "New Title",
            'description': "New Description",
            'translations': [
                {
                    "language_code": "en",
                    "title": "The Great Adventure",
                    "description": "An exciting journey through uncharted lands."  # noqa
                },
                {
                    "language_code": "am",
                    "title": "አስደሳች ጉዞ",
                    "description": "በማይታወቁ መሬቶች ላይ አስገራሚ ጉዞ."
                },
                {
                    "language_code": "or",
                    "title": "Adeemsa Guddaa",
                    "description": "Daandii jajjaboo fi tasgabboof balaa keessa darbuu."  # noqa
                },
                {
                    "language_code": "tg",
                    "title": "Сафари Бузург",
                    "description": "Сафари ҳаяҷоновар тавассути сарзаминҳои номаълум."  # noqa
                }
            ]
        }
        serializer = self.serializer_class(
            data=data, context={'request': self.request})
        self.assertTrue(serializer.is_valid())

        instance = serializer.save()
        self.assertEqual(instance.title, "New Title")
        self.assertEqual(instance.get_lisan_field('title', 'am'), "አስደሳች ጉዞ")
        self.assertEqual(instance.get_lisan_field(
            'description', 'am'), "በማይታወቁ መሬቶች ላይ አስገራሚ ጉዞ.")

    def test_update_with_translations(self):
        """
        Test updating a model instance with translations.
        """
        self.request.method = 'PATCH'
        data = {
            'title': "Updated Title",
            'translations': [
                {'language_code': 'am', 'title': "አዲስ ሰላም"}
            ]
        }
        serializer = self.serializer_class(
            self.model_instance, data=data,
            partial=True, context={'request': self.request})
        self.assertTrue(serializer.is_valid())

        instance = serializer.save()
        self.assertEqual(instance.title, "Updated Title")
        self.assertEqual(instance.get_lisan_field('title', 'am'), "አዲስ ሰላም")

    def test_validation_missing_translations(self):
        """
        Test validation error for missing translations.
        """
        self.request.method = 'POST'
        data = {'title': "New Title"}
        serializer = self.serializer_class(
            data=data, context={'request': self.request})
        with self.assertRaises(ValidationError):
            serializer.is_valid(raise_exception=True)
            serializer.save()

    def test_validation_unsupported_language(self):
        """
        Test validation error for unsupported language code.
        """
        self.request.method = 'POST'
        data = {
            'title': "New Title",
            'translations': [
                {'language_code': 'unsupported', 'title': "Test"}
            ]
        }
        serializer = self.serializer_class(
            data=data, context={'request': self.request})
        with self.assertRaises(ValidationError):
            serializer.is_valid(raise_exception=True)
            serializer.save()

    def test_dynamic_fields_removal(self):
        """
        Test that the `translations` field is removed for non-write methods.
        """
        self.request.method = 'GET'
        serializer = self.serializer_class(context={'request': self.request})
        self.assertNotIn('translations', serializer.fields)

    def test_dynamic_fields_addition(self):
        """
        Test that the `translations` field is added for write methods.
        """
        for method in ['POST', 'PUT', 'PATCH']:
            self.request.method = method
            serializer = self.serializer_class(
                context={'request': self.request})
            self.assertIn('translations', serializer.fields)

    def test_representation_unsupported_language(self):
        """
        Test that the representation falls back to the default language
        when the requested language is not supported.
        """
        self.request.language_code = 'unsupported'
        serializer = self.serializer_class(
            self.model_instance, context={'request': self.request})
        representation = serializer.data

        # Check that it falls back to the default language
        self.assertEqual(representation['title'], "Hello World")
        self.assertEqual(representation['description'], "Sample description")

    def test_validate_translations_partial(self):
        """
        Test validation with partial translations in partial update mode.
        """
        serializer = self.serializer_class(context={'request': self.request})
        translations = [{'language_code': 'en', 'title': "Partial Title"}]
        # No exceptions should be raised for partial updates
        serializer._validate_translations(translations, partial=True)

    def test_validate_translations_missing_languages(self):
        """
        Test validation failure for missing translations in non-partial mode.
        """
        serializer = self.serializer_class(context={'request': self.request})
        translations = [{'language_code': 'en', 'title': "Partial Title"}]
        with self.assertRaises(ValidationError) as context:
            serializer._validate_translations(translations)
        self.assertIn(
            "Missing translations for languages:", str(context.exception))

    def test_create_with_missing_fields(self):
        """
        Test that `create` raises a validation error for missing
        required fields.
        """
        self.request.method = 'POST'
        # Missing `title`
        data = {'translations': [{'language_code': 'am'}]}
        serializer = self.serializer_class(
            data=data, context={'request': self.request})
        with self.assertRaises(ValidationError):
            serializer.is_valid(raise_exception=True)
            serializer.save()

    def test_update_with_unsupported_language(self):
        """
        Test that `update` raises a validation error when an unsupported
        language is used in translations.
        """
        self.request.method = 'PATCH'
        data = {
            'translations': [
                {'language_code': 'unsupported', 'title': "Invalid"}]
        }
        serializer = self.serializer_class(
            self.model_instance, data=data,
            partial=True, context={'request': self.request})

        with self.assertRaises(ValidationError) as context:
            serializer.is_valid(raise_exception=True)
            serializer.save()

        self.assertIn("Unsupported language code", str(context.exception))

    def test_update_with_partial_translations(self):
        """
        Test that `update` allows partial translations with optional fields
        in a PATCH request.
        """
        self.request.method = 'PATCH'
        # Only language_code provided
        data = {
            'translations': [{'language_code': 'am'}]
        }
        serializer = self.serializer_class(
            self.model_instance, data=data, partial=True,
            context={'request': self.request})

        # Validate and save without raising exceptions
        self.assertTrue(serializer.is_valid(raise_exception=True))

        instance = serializer.save()

        # Ensure the instance is updated correctly
        # Unchanged
        self.assertEqual(instance.get_lisan_field('title', 'am'), "ሰላም")
        # Unchanged
        self.assertEqual(instance.get_lisan_field('description', 'am'), "ምሳሌ")

    def test_handle_dynamic_fields_add_translations(self):
        """
        Test that `_handle_dynamic_fields` dynamically adds the `translations`
        field for write methods when not already present.
        """
        self.request.method = 'POST'
        serializer = self.serializer_class(context={'request': self.request})

        # Remove translations field to simulate the condition
        serializer.fields.pop('translations', None)

        # Call _handle_dynamic_fields to add it back
        serializer._handle_dynamic_fields()

        self.assertIn('translations', serializer.fields)

    def test_create_with_unsupported_language(self):
        """
        Test that `create` defaults to the `default_language` if the
        provided language code is not supported.
        """
        self.request.method = 'POST'
        self.request.language_code = 'dkajflskdjfl'
        data = {
            'title': "Unsupported Language Test",
            'translations': [
                {
                    "language_code": "en",
                    "title": "The Great Adventure",
                    "description": "An exciting journey through uncharted lands."  # noqa
                },
                {
                    "language_code": "fr",
                    "title": "አስደሳች ጉዞ",
                    "description": "በማይታወቁ መሬቶች ላይ አስገራሚ ጉዞ."
                },
                {
                    "language_code": "or",
                    "title": "Adeemsa Guddaa",
                    "description": "Daandii jajjaboo fi tasgabboof balaa keessa darbuu."  # noqa
                },
                {
                    "language_code": "tg",
                    "title": "Сафари Бузург",
                    "description": "Сафари ҳаяҷоновар тавассути сарзаминҳои номаълум."  # noqa
                }
            ]
        }
        serializer = self.serializer_class(
            data=data, context={'request': self.request})
        with self.assertRaises(ValidationError):
            serializer.is_valid(raise_exception=True)
            instance = serializer.save()
            # Ensure it defaults to the default language ('en')
            self.assertEqual(
                instance.get_lisan_field('title', 'or'),
                "Unsupported Language Test")
            self.request.language_code = None

    def test_validate_translations_empty_partial(self):
        """
        Test that `_validate_translations` skips validation for
        empty translations in partial updates.
        """
        serializer = self.serializer_class(context={'request': self.request})
        # Should not raise any exception
        serializer._validate_translations([], partial=True)

    def test_validate_translations_empty_non_partial(self):
        """
        Test that `_validate_translations` raises a ValidationError for empty
        translations in non-partial updates.
        """
        serializer = self.serializer_class(context={'request': self.request})
        with self.assertRaises(ValidationError) as context:
            serializer._validate_translations([], partial=False)
        self.assertIn("Translations are required.", str(context.exception))

    def test_validate_translations_missing_fields(self):
        """
        Test that `_validate_translations` raises a ValidationError when
        required fields are missing in translations during non-partial updates.
        """
        serializer = self.serializer_class(context={'request': self.request})
        translations = [{'language_code': 'en'}]  # Missing required fields
        with self.assertRaises(ValidationError) as context:
            serializer._validate_translations(translations, partial=False)
        self.assertIn(
            "Missing translations for languages",
            str(context.exception.detail[0]))

    def test_update_sync_translatable_fields(self):
        """
        Test that updating translatable fields in the main model synchronizes
        changes to the default language translation.
        """
        self.request.method = 'PATCH'
        data = {
            'title': "Updated Title",
            'description': "Updated Description"
        }
        serializer = self.serializer_class(
            self.model_instance, data=data,
            partial=True, context={'request': self.request})
        self.assertTrue(serializer.is_valid())

        instance = serializer.save()

        # Check that the main model fields are updated
        self.assertEqual(instance.title, "Updated Title")
        self.assertEqual(instance.description, "Updated Description")

        # Check that the default language translation is synchronized
        lisan = instance.get_lisan('en')
        self.assertEqual(lisan.title, "Updated Title")
        self.assertEqual(lisan.description, "Updated Description")

    def test_update_sync_translatable_fields_with_language(self):
        """
        Test that updating translatable fields synchronizes changes to the
        specified language translation when the language code is set.
        """
        self.request.method = 'PATCH'
        self.request.language_code = 'am'
        data = {
            'title': "አዲስ ርእስ",
            'description': "አዲስ መግለጫ"
        }
        serializer = self.serializer_class(
            self.model_instance, data=data,
            partial=True, context={'request': self.request})
        self.assertTrue(serializer.is_valid())

        instance = serializer.save()

        # Check that the main model fields are updated
        self.assertEqual(instance.title, "አዲስ ርእስ")
        self.assertEqual(instance.description, "አዲስ መግለጫ")

        # Check that the 'am' translation is updated
        lisan = instance.get_lisan('am')
        self.assertEqual(lisan.title, "አዲስ ርእስ")
        self.assertEqual(lisan.description, "አዲስ መግለጫ")

    def test_partial_update_without_translatable_fields(self):
        """
        Test that updating non-translatable fields does not affect
        translations.
        """
        self.request.method = 'PATCH'
        data = {'author': "New Author"}
        serializer = self.serializer_class(
            self.model_instance, data=data,
            partial=True, context={'request': self.request})
        self.assertTrue(serializer.is_valid())

        instance = serializer.save()

        # Check that the main model field is updated
        self.assertEqual(instance.author, "New Author")

        # Check that translations remain unchanged
        lisan = instance.get_lisan('am')
        self.assertEqual(lisan.title, "ሰላም")
        self.assertEqual(lisan.description, "ምሳሌ")

    def test_update_translatable_and_non_translatable_fields(self):
        """
        Test that updating both translatable and non-translatable fields
        synchronizes translatable fields while updating the main model.
        """
        self.request.method = 'PATCH'
        data = {
            'title': "Updated Title",
            'author': "New Author"
        }
        serializer = self.serializer_class(
            self.model_instance, data=data,
            partial=True, context={'request': self.request})
        self.assertTrue(serializer.is_valid())

        instance = serializer.save()

        # Check that both main model fields are updated
        self.assertEqual(instance.title, "Updated Title")
        self.assertEqual(instance.author, "New Author")

        # Check that the default language translation is synchronized
        lisan = instance.get_lisan('en')
        self.assertEqual(lisan.title, "Updated Title")
        self.assertEqual(lisan.description, "")

    def test_update_with_unsupported_language_to_sync(self):
        """
        Test that updating with an unsupported language defaults to the
        default language for synchronization.
        """
        self.request.method = 'PATCH'
        self.request.language_code = 'unsupported'
        data = {
            'title': "Unsupported Language Title"
        }
        serializer = self.serializer_class(
            self.model_instance, data=data,
            partial=True, context={'request': self.request})
        self.assertTrue(serializer.is_valid())

        instance = serializer.save()

        # Check that the default language is used for synchronization
        lisan = instance.get_lisan('en')
        self.assertEqual(lisan.title, "Unsupported Language Title")



# google_translate_service.py
# from googletrans import Translator
from lisan.translation_services import BaseTranslationService


class GoogleTranslateService(BaseTranslationService):
    def __init__(self):
        self.translator = lambda a, b: f"{a} in {b}"

    def translate(self, text, target_language):
        return self.translator(text, target_language)



from django.test import TestCase, RequestFactory
from django.contrib.admin.sites import AdminSite
from django.db import models
from django.contrib import admin
from lisan.admin import LisanAdminMixin
from tests.models import TestModel, SomeModel
from django.core.exceptions import ImproperlyConfigured


class TestLisanAdminMixin(TestCase):
    """
    Test suite for the LisanAdminMixin class, which provides multilingual
    support in the Django admin interface for models using LisanModelMixin.
    """

    def setUp(self):
        """
        Set up the test environment. Create a test model instance and an
        admin instance for the tests.
        """
        self.factory = RequestFactory()
        self.site = AdminSite()

        # Create a test admin instance for the TestModel
        class TestModelAdmin(LisanAdminMixin, admin.ModelAdmin):
            model = TestModel

        self.admin = TestModelAdmin(model=TestModel, admin_site=self.site)
        self.instance = TestModel.objects.create(
            title="Hello World",
            description="Sample description"
        )

    def test_admin_initialization(self):
        """
        Test that the admin mixin dynamically generates getter methods
        for all fields defined in the model's `lisan_fields` and adds
        them to the `list_display`.
        """
        self.assertIn('get_lisan_title', self.admin.list_display)
        self.assertIn('get_lisan_description', self.admin.list_display)

    def test_generated_getter_methods(self):
        """
        Test that getter methods for multilingual fields are dynamically
        added to the admin and correctly return the localized content.
        """
        # Check that the getter methods exist
        self.assertTrue(hasattr(self.admin, 'get_lisan_title'))
        self.assertTrue(hasattr(self.admin, 'get_lisan_description'))

        # Check the behavior of the getter methods
        getter_title = getattr(self.admin, 'get_lisan_title')
        self.assertEqual(getter_title(self.instance), "Hello World")

        getter_description = getattr(self.admin, 'get_lisan_description')
        self.assertEqual(
            getter_description(self.instance), "Sample description")

    def test_display_format(self):
        """
        Test that the dynamically generated getter methods have the correct
        short_description format for display in the admin interface.
        """
        getter_title = getattr(self.admin, 'get_lisan_title')
        self.assertEqual(
            getter_title.short_description, "Title (EN)")

        getter_description = getattr(self.admin, 'get_lisan_description')
        self.assertEqual(
            getter_description.short_description, "Description (EN)")

    def test_get_lisan_inline(self):
        """
        Test the creation of the inline class for managing translations
        in the admin interface.
        """
        inline_class = self.admin.get_lisan_inline()
        self.assertTrue(issubclass(inline_class, admin.TabularInline))
        self.assertEqual(inline_class.model, TestModel.Lisan)

    def test_get_lisan_inline_no_lisan_model(self):
        """
        Test that get_lisan_inline raises ImproperlyConfigured if no
        related Lisan model exists.
        """
        class NoLisanModel:
            _meta = SomeModel._meta  # Simulate a model with no lisan_fields

        class NoLisanAdmin(LisanAdminMixin, admin.ModelAdmin):
            model = NoLisanModel

        _admin = NoLisanAdmin(model=NoLisanModel, admin_site=None)

        with self.assertRaises(ImproperlyConfigured) as context:
            _admin.get_lisan_inline()
        self.assertIn(
            "NoLisanModel does not have a related Lisan model.",
            str(context.exception)
        )

    def test_get_inlines(self):
        """
        Test that the admin correctly includes the inline class for managing
        translations when `lisan_fields` is defined in the model.
        """
        # Verify that the inline class is returned for models with lisan_fields
        inlines = self.admin.get_inlines(None)
        self.assertEqual(len(inlines), 1)
        self.assertTrue(issubclass(inlines[0], admin.TabularInline))

        # Verify that no inlines are returned for models without lisan_fields
        class NoLisanModel(models.Model):
            pass

        class NoLisanModelAdmin(LisanAdminMixin, admin.ModelAdmin):
            model = NoLisanModel

        no_lisan_admin = NoLisanModelAdmin(
            model=NoLisanModel, admin_site=self.site)
        self.assertEqual(no_lisan_admin.get_inlines(None), [])

    def test_model_without_lisan_fields(self):
        """
        Test that the admin mixin does not add any getters or modify
        `list_display` for models that do not define `lisan_fields`.
        """
        class NoLisanModel(models.Model):
            name = models.CharField(max_length=100)

        class NoLisanModelAdmin(LisanAdminMixin, admin.ModelAdmin):
            model = NoLisanModel

        admin_instance = NoLisanModelAdmin(
            model=NoLisanModel, admin_site=self.site)

        # Ensure no getters are added
        self.assertFalse(hasattr(admin_instance, 'get_lisan_name'))

    def test_empty_lisan_fields(self):
        """
        Test the behavior of the admin mixin when the model's `lisan_fields`
        is an empty list.
        """
        class EmptyLisanModel(models.Model):
            lisan_fields = []

        class EmptyLisanModelAdmin(LisanAdminMixin, admin.ModelAdmin):
            model = EmptyLisanModel

        admin_instance = EmptyLisanModelAdmin(
            model=EmptyLisanModel, admin_site=self.site)

        # Ensure no lisan getter methods are added to list_display
        lisan_getters = [field for field in admin_instance.list_display if field.startswith('get_lisan_')] # noqa
        self.assertEqual(lisan_getters, [])



from django.db import models

SECRET_KEY = 'some-secret-key'

INSTALLED_APPS = [
    'django.contrib.contenttypes',
    'django.contrib.auth',
    'tests',  # Include the tests module if it contains models
]

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}

LANGUAGE_CODE = 'en-us'

DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"

# Lisan Settings
LISAN_DEFAULT_LANGUAGE = 'en'
LISAN_ALLOWED_LANGUAGES = ['en', 'am', 'or', 'tg']
LISAN_PRIMARY_KEY_TYPE = models.UUIDField
LISAN_DEFAULT_TRANSLATION_SERVICE = 'tests.google_translate_service.GoogleTranslateService' # noqa

MIDDLEWARE = [
    'django.middleware.common.CommonMiddleware',
    'lisan.middleware.LanguageMiddleware',
]



from django.conf import settings
from django.test import TestCase
from django.core.exceptions import ValidationError
from django.db import IntegrityError, models
from lisan.mixins import LisanModelMixin
from lisan.metaclasses import create_lisan_model  # LisanModelMeta
from tests.models import TestModel


class MetaclassesTestCase(TestCase):
    def setUp(self):
        # Update settings for testing
        settings.LISAN_PRIMARY_KEY_TYPE = models.UUIDField
        self.fields = {
            'title': models.CharField(max_length=100, blank=True, default=''),
            'description': models.TextField(blank=True, default='')
        }
        self.instance = TestModel.objects.create(
            title="Hello World",
            description="Sample description"
        )

    # Test the initial creation of a model
    def test_initial_creation(self):
        self.assertEqual(self.instance.title, "Hello World")
        self.assertEqual(self.instance.description, "Sample description")

    # Test setting and retrieving translations
    def test_set_and_get_translation(self):
        self.instance.set_lisan(
            'am', title="ሰላም ለዓለም", description="ምሳሌ መግለጫ")
        self.assertEqual(
            self.instance.get_lisan_field('title', 'am'), "ሰላም ለዓለም")
        self.assertEqual(self.instance.get_lisan_field(
            'description', 'am'), "ምሳሌ መግለጫ")

    # Test fallback behavior when translation is missing
    def test_fallback_language(self):
        # Falls back to default
        self.assertEqual(
            self.instance.get_lisan_field('title', 'am'), "Hello World")

    # Test overwriting a translation
    def test_overwriting_translation(self):
        self.instance.set_lisan('tg', title="ሰላም አለም")
        self.instance.set_lisan('tg', title="ሰላም ዓለም")
        self.assertEqual(self.instance.get_lisan_field(
            'title', 'tg'), "ሰላም ዓለም")

    # Test retrieving translations for invalid language
    def test_invalid_language_code(self):
        with self.assertRaises(ValueError):
            self.instance.set_lisan(None, title="Invalid")

    def test_unsupported_language_code(self):
        with self.assertRaises(ValueError):
            self.instance.set_lisan('abcldkjfd', title="Invalid")

    # Test setting a translation with invalid data
    def test_invalid_data_for_translation(self):
        with self.assertRaises(IntegrityError):
            self.instance.set_lisan('am', title=None)

    def test_create_lisan_model(self):
        lisan_model = create_lisan_model(
            TestModel, self.fields, models.UUIDField)

        # Assert the model has the required fields
        self.assertTrue(hasattr(lisan_model, 'language_code'))
        self.assertTrue(hasattr(lisan_model, 'id'))
        self.assertTrue(hasattr(lisan_model, 'title'))
        self.assertTrue(hasattr(lisan_model, 'description'))

        # Assert the ForeignKey relationship
        self.assertTrue(hasattr(lisan_model, 'testmodel'))

        # Assert the primary key is UUIDField
        self.assertEqual(
            lisan_model._meta.pk.__class__, models.UUIDField)

        # Assert the unique constraint
        constraints = lisan_model._meta.constraints
        self.assertEqual(len(constraints), 1)
        self.assertEqual(
            constraints[0].fields, tuple(['language_code', 'testmodel']))

        # Assert the table name
        self.assertEqual(lisan_model._meta.db_table, 'tests_testmodel_lisan')

    def test_missing_lisan_fields(self):
        with self.assertRaises(AttributeError):
            class InvalidModel(LisanModelMixin, models.Model):
                a = models.CharField(max_length=100)

    def test_invalid_lisan_fields(self):
        with self.assertRaises(ValueError):
            class InvalidModel(LisanModelMixin, models.Model):
                lisan_fields = ['nonexistent_field']

    def test_invalid_primary_key_type(self):
        with self.assertRaises(TypeError):
            create_lisan_model(TestModel, self.fields, primary_key_type=str)

    def test_default_primary_key_type(self):
        lisan_model = create_lisan_model(TestModel, self.fields)
        self.assertEqual(
            lisan_model._meta.pk.__class__, models.BigAutoField)

    def test_duplicate_translation(self):
        lisan_model = create_lisan_model(
            TestModel, self.fields, models.UUIDField)
        lisan_model.objects.create(
            language_code='am',
            title="Sample",
            description="Description",
            testmodel_id=self.instance.id
        )

        with self.assertRaises(ValidationError):
            duplicate_instance = lisan_model(
                language_code='am',
                title="Duplicate",
                description="Duplicate description",
                testmodel_id=self.instance.id
            )
            duplicate_instance.clean()

    def test_missing_model_class(self):
        with self.assertRaises(AttributeError):
            create_lisan_model(None, self.fields)

    def test_empty_fields(self):
        lisan_model = create_lisan_model(TestModel, {}, models.UUIDField)
        self.assertFalse(hasattr(lisan_model, 'title'))
        self.assertFalse(hasattr(lisan_model, 'description'))

    # def test_invalid_foreign_key_setup(self):
    #     with self.assertRaises(ValueError):
    #         class InvalidForeignKeyModel(
    #                 models.Model, metaclass=LisanModelMeta):
    #             lisan_fields = ['title']
    #             title = models.CharField(max_length=100)
    #             lisan_primary_key_type = "invalid_type"



from django.db import models
from lisan import LisanModelMixin


class SomeModel(models.Model):
    title = models.CharField(max_length=100, blank=True, default='')
    description = models.TextField(blank=True, default='')


class TestModel(LisanModelMixin, models.Model):
    lisan_fields = ['title', 'description']
    title = models.CharField(max_length=100, blank=True, default='')
    description = models.TextField(blank=True, default='')
    some = models.ForeignKey(
        SomeModel, null=True, blank=True, on_delete=models.CASCADE)
    author = models.CharField(
        max_length=100, blank=True, null=True, default='')


class TheOtherModel(models.Model):
    name = models.CharField(max_length=100, blank=True, default='')
    title = models.CharField(max_length=100, blank=True, default='')
    model_link = models.ForeignKey(
        TestModel, null=True, blank=True, on_delete=models.CASCADE)
